﻿/**
 *       Project: ServerLib
 *     Copyright: Copyright (C) 2008, Rob Kennedy
 *   Create Date: 2008-14-07 11:13 PM
 * Modified Date: 2008-14-07 11:13 PM    RK  Added comments
 *      Comments: Ignore SA1633.
 *      
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace ServerLib.Data
{
    using System.Data;

    /// --------------------------------------------------------------------
    /// Project	 : ServerLib
    /// Class	 : Logging
    /// Inherits : IDisposable
    /// --------------------------------------------------------------------
    /// <summary>
    ///  Database interface class used to instantiate a IDatabase inherited
    ///  class.
    /// </summary>
    /// <authorname>Rob Kennedy</authorname>
    /// --------------------------------------------------------------------
    public interface IDatabase
    {
        /// <summary>Gets the unique database class interface name</summary>
        string Name
        {
            get;
        }

        /// <summary>Gets or sets the database connection string data</summary>
        ConnectionStringBuilder ConnectionString 
        { 
            get; 
            set; 
        }

        /// <summary>Gets a value indicating whether the database connection is connected</summary>
        bool IsConnected 
        { 
            get; 
        }

        /// <summary>Gets any database error message returned</summary>
        string ErrorMessage 
        { 
            get; 
        }

        /// <summary>
        /// Opens a database connection
        /// </summary>
        /// <returns>True if the database connection was a success</returns>
        bool Open();

        /// <summary>
        /// Opens a database connection
        /// </summary>
        /// <param name="connectionString">A connection string to use for opening</param>
        /// <returns>True if the database connection was a success</returns>
        bool Open(string connectionString);
        
        /// <summary>Closes a database connection</summary>
        void Close();

        /// <summary>
        /// Executes a SQL command and returns results in a DataTable object 
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <returns>If no error occurs a DataTable object is returned, otherwise null on error</returns>
        DataTable SelectQuery(string sqlCommand);

        /// <summary>
        /// Executes a SQL command and returns results in a DataTable object 
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>If no error occurs a DataTable object is returned, otherwise null on error</returns>
        DataTable SelectQuery(string sqlCommand, params object[] parameterValues);

        /// <summary>
        /// Executes a SQL command and returns a single row in a DataRow object
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <returns>If no error occurs a DataRow object is returned, otherwise null on error</returns>
        DataRow SelectQueryRow(string sqlCommand);

        /// <summary>
        /// Executes a SQL command and returns a single row in a DataRow object
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="rowIndex">A specific row index of a number of expected row results to return</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>If no error occurs a DataRow object is returned, otherwise null on error</returns>
        DataRow SelectQueryRow(string sqlCommand, int rowIndex, params object[] parameterValues);

        /// <summary>
        /// Executes a SQL command and returns the top most row in a DataRow object
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <returns>If no error occurs a DataRow object is returned, otherwise null on error</returns>
        DataRow SelectQueryTopRow(string sqlCommand);

        /// <summary>
        /// Executes a SQL command and returns the top most row in a DataRow object
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>If no error occurs a DataRow object is returned, otherwise null on error</returns>
        DataRow SelectQueryTopRow(string sqlCommand, params object[] parameterValues);

        /// <summary>
        /// Executes a SQL command and returns a boolean value based on either the result or the record
        /// count of results.
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="useRecordCount">If true, the check will use the query results record count for results</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>Returns true if the SQL command resulted in a true state or returned one or more rows</returns>
        bool QueryCheck(string sqlCommand, bool useRecordCount, params object[] parameterValues);

        /// <summary>
        /// Executes a SQL command and returns the number of rows effected.
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>Returns the number of rows effected by the SQL command</returns>
        int ExecuteQuery(string sqlCommand, params object[] parameterValues);

        /// <summary>
        /// Executes a SQL command and returns the value of the specified field in the results.
        /// </summary>
        /// <param name="sqlCommand">The SQL command to execute</param>
        /// <param name="valueField">The name of the result field to retrive a value from</param>
        /// <param name="parameterValues">Optional parameters to insert into the SQL command string</param>
        /// <returns>Returns the value of the specified field in the top most row of any data results or null on no results</returns>
        object QueryValue(string sqlCommand, string valueField, params object[] parameterValues);

        /// <summary>
        /// Retrieves the current id of any identity column in the data table specified.
        /// </summary>
        /// <param name="tableName">The table name to retrieve an identity value from</param>
        /// <returns>Returns the current identity value of a table, 0 if no identity exists.</returns>
        int GetCurrentIdentity(string tableName);
    }
}
