﻿/**
 *       Project: ServerLib
 *     Copyright: Copyright (C) 2008, Rob Kennedy
 *   Create Date: 2008-07-15 3:40 PM
 * Modified Date: 2008-07-15 3:40 PM    RK  Added comments
 *      Comments: Ignore SA1633.
 *      
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

namespace ServerLib.Settings
{
    using System.Collections.ObjectModel;
    using System.Data;
    using ServerLib.Core;
    using ServerLib.Data;

    /// --------------------------------------------------------------------
    /// Project	 : ServerLib
    /// Class	 : Alias
    /// Inherits : DataRowRecord
    /// --------------------------------------------------------------------
    /// <summary>
    /// A class representation of an Alias database record
    /// </summary>
    /// <authorname>Rob Kennedy</authorname>
    /// <remarks></remarks>
    /// --------------------------------------------------------------------
    public class Alias : DataRowRecord
    {
        #region Private Properties
        /// <summary>
        /// Contains a collection list of additionally associated users (mailing lists)
        /// </summary>
        private Collection<User> additionalUsers = new Collection<User>();

        /// <summary>
        /// Contains a collection list of mailing list e-mail addresses
        /// </summary>
        private Collection<string> mailingList = new Collection<string>();
        #endregion

        #region Public Properties

        /// <summary>
        /// Gets the unique Alias record Identifier number
        /// </summary>
        public int AliasID
        {
            get
            {
                return Types.FixInteger(this.InternalDataRow["AliasID"]);
            }
        }

        /// <summary>
        /// Gets or sets the domain id of the owner domain
        /// </summary>
        public int DomainID
        {
            get
            {
                return Types.FixInteger(this.InternalDataRow["DomainID"]);
            }

            set
            {
                if (this.InternalDataRow.RowState == System.Data.DataRowState.Unchanged)
                {
                    this.InternalDataRow.BeginEdit();
                }

                this.InternalDataRow["DomainID"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the Alias name value
        /// </summary>
        public string AliasName
        {
            get
            {
                return Types.FixString(this.InternalDataRow["AliasName"]);
            }

            set
            {
                if (this.InternalDataRow.RowState == System.Data.DataRowState.Unchanged)
                {
                    this.InternalDataRow.BeginEdit();
                }

                this.InternalDataRow["AliasName"] = value;
            }
        }

        /// <summary>
        /// Gets or sets the unique user id for the alias owner
        /// </summary>
        public int UserID
        {
            get
            {
                return Types.FixInteger(this.InternalDataRow["UserID"]);
            }

            set
            {
                if (this.InternalDataRow.RowState == System.Data.DataRowState.Unchanged)
                {
                    this.InternalDataRow.BeginEdit();
                }

                this.InternalDataRow["UserID"] = value;
            }
        }

        /// <summary>
        /// Gets a collection list of associated users with this alias
        /// </summary>
        public Collection<User> AdditionalUsers
        {
            get
            {
                if (this.Connection != null)
                {
                    this.additionalUsers.Clear();

                    bool wasPreviouslyOpen = this.Connection.IsConnected;

                    if (!wasPreviouslyOpen)
                    {
                        this.Connection.Open();
                    }

                    if (this.Connection.IsConnected)
                    {
                        DataTable resultsTable = this.Connection.SelectQuery("SELECT Users.* FROM Users INNER JOIN AliasUserAssoc ON Users.UserID=AliasUserAssoc.UserID WHERE AliasUserAssoc.AliasID={0}", this.AliasID);

                        if (resultsTable != null)
                        {
                            foreach (DataRow userRow in resultsTable.Rows)
                            {
                                this.additionalUsers.Add(new User(userRow));
                            }
                        }
                    }

                    if (!wasPreviouslyOpen)
                    {
                        this.Connection.Close();
                    }
                }

                return this.additionalUsers;
            }
        }

        /// <summary>
        /// Gets a collection list of mailing list e-mail addresses for this alias
        /// </summary>
        public Collection<string> MailingList
        {
            get
            {
                if (this.Connection != null)
                {
                    this.mailingList.Clear();

                    bool wasPreviouslyOpen = this.Connection.IsConnected;

                    if (!wasPreviouslyOpen)
                    {
                        this.Connection.Open();
                    }

                    if (this.Connection.IsConnected)
                    {
                        DataTable resultsTable = this.Connection.SelectQuery(SqlCommandStringBuilder.BuildSelect("MailingList", new string[] { "SendToAddress" }, string.Format("AliasID={0}", this.AliasID), string.Empty));

                        if (resultsTable != null)
                        {
                            foreach (DataRow addrRow in resultsTable.Rows)
                            {
                                string address = Types.FixString(addrRow["SendToAddress"]);
                                if (address.Length > 0)
                                {
                                    this.mailingList.Add(address);
                                }
                            }
                        }
                    }

                    if (!wasPreviouslyOpen)
                    {
                        this.Connection.Close();
                    }
                }

                return this.mailingList;
            }
        }
        #endregion

        #region Public Constructors

        /// <summary>
        /// Initializes the Alias object with a new DataRow class
        /// </summary>
        /// <param name="dataConnection">The IDatabase object to connect with</param>
        public Alias(IDatabase dataConnection)
            : base(dataConnection)
        {
        }

        /// <summary>
        /// Initializes the Alias object with a new DataRow class
        /// </summary>
        /// <param name="dataConnection">The IDatabase object to connect with</param>
        /// <param name="tableName">The Record table name to load into the Record object</param>
        /// <param name="fieldName">The Record table field name name used to identify when loading the recordId</param>
        /// <param name="recordId">The unique record ID number to load into the Record object</param>
        public Alias(IDatabase dataConnection, string tableName, string fieldName, int recordId)
            : base(dataConnection, tableName, fieldName, recordId)
        {
        }

        /// <summary>
        /// Initializes the Alias object with a new DataRow class
        /// </summary>
        /// <param name="dataTable">The DataTable object to retrieve a new DataRow object from</param>
        public Alias(DataTable dataTable)
            : base(dataTable)
        {
        }
        
        /// <summary>
        /// Initializes the Alias object with a new DataRow class
        /// </summary>
        /// <param name="dataRow">The DataRow object to initialize the Domain object with</param>
        public Alias(DataRow dataRow)
            : base(dataRow)
        {
        }
        #endregion
    }
}
